@@ -1,9 +1,8 @@
 """Generate langauge specific loaders for a particular SALAD schema."""
 import sys
-from typing import Any, Dict, List, MutableMapping, Optional, Union
+from typing import Any, Dict, List, MutableMapping, Optional
 
 from typing_extensions import Text  # pylint: disable=unused-import
-# move to a regular typing import when Python 3.3-3.6 is no longer supported
 
 from . import schema
 from .codegen_base import CodeGenBase
@@ -13,12 +12,15 @@ from .ref_resolver import Loader  # pylint: disable=unused-import
 from .schema import shortname
 from .utils import aslist
 
+# move to a regular typing import when Python 3.3-3.6 is no longer supported
 
-def codegen(lang,             # type: str
-            i,                # type: List[Dict[Text, Any]]
-            schema_metadata,  # type: Dict[Text, Any]
-            loader            # type: Loader
-           ):  # type: (...) -> None
+
+def codegen(
+    lang,  # type: str
+    i,  # type: List[Dict[Text, Any]]
+    schema_metadata,  # type: Dict[Text, Any]
+    loader,  # type: Loader
+):  # type: (...) -> None
     """Generate classes with loaders for the given Schema Salad description."""
 
     j = schema.extend_and_specialize(i, loader)
@@ -29,7 +31,7 @@ def codegen(lang,             # type: str
     elif lang == "java":
         gen = JavaCodeGen(schema_metadata.get("$base", schema_metadata.get("id")))
     else:
-        raise Exception("Unsupported code generation language '%s'" % lang)
+        raise Exception("Unsupported code generation language '{}'".format(lang))
     assert gen is not None
 
     gen.prologue()
@@ -59,16 +61,26 @@ def codegen(lang,             # type: str
                 if field.get("jsonldPredicate") == "@id":
                     idfield = field.get("name")
 
-            gen.begin_class(rec["name"], aslist(rec.get("extends", [])), rec.get("doc", ""),
-                            rec.get("abstract", False), field_names, idfield)
+            gen.begin_class(
+                rec["name"],
+                aslist(rec.get("extends", [])),
+                rec.get("doc", ""),
+                rec.get("abstract", False),
+                field_names,
+                idfield,
+            )
             gen.add_vocab(shortname(rec["name"]), rec["name"])
 
             for field in rec.get("fields", []):
                 if field.get("jsonldPredicate") == "@id":
                     fieldpred = field["name"]
                     optional = bool("https://w3id.org/cwl/salad#null" in field["type"])
-                    uri_loader = gen.uri_loader(gen.type_loader(field["type"]), True, False, None)
-                    gen.declare_id_field(fieldpred, uri_loader, field.get("doc"), optional)
+                    uri_loader = gen.uri_loader(
+                        gen.type_loader(field["type"]), True, False, None
+                    )
+                    gen.declare_id_field(
+                        fieldpred, uri_loader, field.get("doc"), optional
+                    )
                     break
 
             for field in rec.get("fields", []):
@@ -82,15 +94,22 @@ def codegen(lang,             # type: str
                     if jld.get("typeDSL"):
                         type_loader = gen.typedsl_loader(type_loader, ref_scope)
                     elif jld.get("_type") == "@id":
-                        type_loader = gen.uri_loader(type_loader, jld.get("identity", False),
-                                                     False, ref_scope)
+                        type_loader = gen.uri_loader(
+                            type_loader, jld.get("identity", False), False, ref_scope
+                        )
                     elif jld.get("_type") == "@vocab":
-                        type_loader = gen.uri_loader(type_loader, False, True, ref_scope)
+                        type_loader = gen.uri_loader(
+                            type_loader, False, True, ref_scope
+                        )
 
                     map_subject = jld.get("mapSubject")
                     if map_subject:
                         type_loader = gen.idmap_loader(
-                            field["name"], type_loader, map_subject, jld.get("mapPredicate"))
+                            field["name"],
+                            type_loader,
+                            map_subject,
+                            jld.get("mapPredicate"),
+                        )
 
                     if "_id" in jld and jld["_id"][0] != "@":
                         fieldpred = jld["_id"]
@@ -103,9 +122,6 @@ def codegen(lang,             # type: str
             gen.end_class(rec["name"], field_names)
 
     root_type = list(document_roots)
-    root_type.append({
-        "type": "array",
-        "items": document_roots
-    })
+    root_type.append({"type": "array", "items": document_roots})
 
     gen.epilogue(gen.type_loader(root_type))
