@@ -2,10 +2,16 @@
 
 from __future__ import absolute_import
 
+import os
 import shutil
+import sys
 import tempfile
 
 import pytest  # type: ignore
+from requests import Session
+
+from schema_salad.ref_resolver import DefaultFetcher, Loader, file_uri
+from schema_salad.tests.util import get_data
 
 
 @pytest.fixture
@@ -15,24 +21,19 @@ def tmp_dir_fixture(request):
     @request.addfinalizer
     def teardown():
         shutil.rmtree(d)
+
     return d
 
-def test_Loader_initialisation_for_HOME_env_var(tmp_dir_fixture):
-    import os
-    from schema_salad.ref_resolver import Loader
-    from requests import Session
 
+def test_Loader_initialisation_for_HOME_env_var(tmp_dir_fixture):
     # Ensure HOME is set.
     os.environ["HOME"] = tmp_dir_fixture
 
     loader = Loader(ctx={})
     assert isinstance(loader.session, Session)
 
-def test_Loader_initialisation_for_TMP_env_var(tmp_dir_fixture):
-    import os
-    from schema_salad.ref_resolver import Loader
-    from requests import Session
 
+def test_Loader_initialisation_for_TMP_env_var(tmp_dir_fixture):
     # Ensure HOME is missing.
     if "HOME" in os.environ:
         del os.environ["HOME"]
@@ -42,11 +43,8 @@ def test_Loader_initialisation_for_TMP_env_var(tmp_dir_fixture):
     loader = Loader(ctx={})
     assert isinstance(loader.session, Session)
 
-def test_Loader_initialisation_with_neither_TMP_HOME_set(tmp_dir_fixture):
-    import os
-    from schema_salad.ref_resolver import Loader
-    from requests import Session
 
+def test_Loader_initialisation_with_neither_TMP_HOME_set(tmp_dir_fixture):
     # Ensure HOME is missing.
     if "HOME" in os.environ:
         del os.environ["HOME"]
@@ -56,12 +54,8 @@ def test_Loader_initialisation_with_neither_TMP_HOME_set(tmp_dir_fixture):
     loader = Loader(ctx={})
     assert isinstance(loader.session, Session)
 
-def test_DefaultFetcher_urljoin_win32(tmp_dir_fixture):
-    import os
-    import sys
-    from schema_salad.ref_resolver import DefaultFetcher
-    from requests import Session
 
+def test_DefaultFetcher_urljoin_win32(tmp_dir_fixture):
     # Ensure HOME is set.
     os.environ["HOME"] = tmp_dir_fixture
 
@@ -95,7 +89,9 @@ def test_DefaultFetcher_urljoin_win32(tmp_dir_fixture):
         assert url == "file:///d:/foo/soup.cwl"
 
         # resolving absolute non-drive URIs still works
-        url = fetcher.urljoin("file:///C:/Users/fred/foo.cwl", "http://example.com/bar/soup.cwl")
+        url = fetcher.urljoin(
+            "file:///C:/Users/fred/foo.cwl", "http://example.com/bar/soup.cwl"
+        )
         assert url == "http://example.com/bar/soup.cwl"
         # and of course relative paths from http://
         url = fetcher.urljoin("http://example.com/fred/foo.cwl", "soup.cwl")
@@ -105,10 +101,11 @@ def test_DefaultFetcher_urljoin_win32(tmp_dir_fixture):
         url = fetcher.urljoin("http://example.com/fred/foo.cwl", "/bar/soup.cwl")
         assert url == "http://example.com/bar/soup.cwl"
 
-
         # Security concern - can't resolve file: from http:
         with pytest.raises(ValueError):
-            url = fetcher.urljoin("http://example.com/fred/foo.cwl", "file:///c:/bar/soup.cwl")
+            url = fetcher.urljoin(
+                "http://example.com/fred/foo.cwl", "file:///c:/bar/soup.cwl"
+            )
         # Drive-relative -- should NOT return "absolute" URI c:/bar/soup.cwl"
         # as that is a potential remote exploit
         with pytest.raises(ValueError):
@@ -117,12 +114,8 @@ def test_DefaultFetcher_urljoin_win32(tmp_dir_fixture):
     finally:
         sys.platform = actual_platform
 
-def test_DefaultFetcher_urljoin_linux(tmp_dir_fixture):
-    import os
-    import sys
-    from schema_salad.ref_resolver import DefaultFetcher
-    from requests import Session
 
+def test_DefaultFetcher_urljoin_linux(tmp_dir_fixture):
     # Ensure HOME is set.
     os.environ["HOME"] = tmp_dir_fixture
 
@@ -140,7 +133,9 @@ def test_DefaultFetcher_urljoin_linux(tmp_dir_fixture):
         url = fetcher.urljoin("file:///home/fred/foo.cwl", "/baz/soup.cwl")
         assert url == "file:///baz/soup.cwl"
 
-        url = fetcher.urljoin("file:///home/fred/foo.cwl", "http://example.com/bar/soup.cwl")
+        url = fetcher.urljoin(
+            "file:///home/fred/foo.cwl", "http://example.com/bar/soup.cwl"
+        )
         assert url == "http://example.com/bar/soup.cwl"
 
         url = fetcher.urljoin("http://example.com/fred/foo.cwl", "soup.cwl")
@@ -152,10 +147,69 @@ def test_DefaultFetcher_urljoin_linux(tmp_dir_fixture):
 
         # Security concern - can't resolve file: from http:
         with pytest.raises(ValueError):
-            url = fetcher.urljoin("http://example.com/fred/foo.cwl", "file:///bar/soup.cwl")
+            url = fetcher.urljoin(
+                "http://example.com/fred/foo.cwl", "file:///bar/soup.cwl"
+            )
 
         # But this one is not "dangerous" on Linux
         fetcher.urljoin("http://example.com/fred/foo.cwl", "c:/bar/soup.cwl")
 
     finally:
         sys.platform = actual_platform
+
+
+def test_import_list():
+    import schema_salad.ref_resolver
+    from schema_salad.sourceline import cmap
+
+    basedir = schema_salad.ref_resolver.file_uri(os.path.dirname(__file__) + "/")
+    loader = schema_salad.ref_resolver.Loader({})
+    ra, _ = loader.resolve_all(cmap({u"foo": {"$import": "list.json"}}), basedir)
+
+    assert {u"foo": ["bar", "baz"]} == ra
+
+
+def test_fetch_inject_id():
+    l1 = Loader({"id": "@id"})
+    furi1 = file_uri(get_data("schema_salad/tests/inject-id1.yml")).lower()
+    r1, _ = l1.resolve_ref(furi1)
+    assert {"id": furi1 + "#foo", "bar": "baz"} == r1
+    assert [furi1, furi1 + "#foo"] == sorted(list(k.lower() for k in l1.idx.keys()))
+
+    l2 = Loader({"id": "@id"})
+    furi2 = file_uri(get_data("schema_salad/tests/inject-id2.yml")).lower()
+    r2, _ = l2.resolve_ref(furi2)
+    assert {"id": furi2, "bar": "baz"} == r2
+    assert [furi2] == sorted(list(k.lower() for k in l2.idx.keys()))
+
+    l3 = Loader({"id": "@id"})
+    furi3 = file_uri(get_data("schema_salad/tests/inject-id3.yml")).lower()
+    r3, _ = l3.resolve_ref(furi3)
+    assert {"id": "http://example.com", "bar": "baz"} == r3
+    assert [furi3, "http://example.com"] == sorted(
+        list(k.lower() for k in l3.idx.keys())
+    )
+
+
+def test_attachments():
+    furi = file_uri(get_data("schema_salad/tests/multidoc.yml"))
+
+    l1 = Loader({})
+    r1, _ = l1.resolve_ref(furi)
+    with open(get_data("schema_salad/tests/multidoc.yml"), "rt") as f:
+        content = f.read()
+        assert {"foo": "bar", "baz": content, "quux": content} == r1
+
+    l2 = Loader({}, allow_attachments=lambda x: x["foo"] == "bar")
+    r2, _ = l2.resolve_ref(furi)
+    assert {
+        "foo": "bar",
+        "baz": "This is the {first attachment}.\n",
+        "quux": "This is the [second attachment].",
+    } == r2
+
+    l3 = Loader({}, allow_attachments=lambda x: x["foo"] == "baz")
+    r3, _ = l3.resolve_ref(furi)
+    with open(get_data("schema_salad/tests/multidoc.yml"), "rt") as f:
+        content = f.read()
+        assert {"foo": "bar", "baz": content, "quux": content} == r3
